package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.strike;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.api.server.HttpServer;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.util.IteratorUtil;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.Teams;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.VaroTeam;
import io.papermc.paper.ban.BanListType;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.OfflinePlayer;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

public class Strike extends Appliance {
    public Strike() {
        Bukkit.getScheduler().scheduleSyncRepeatingTask(
            Main.instance(),
            CordLeak::update,
            Ticks.TICKS_PER_SECOND,
            Ticks.TICKS_PER_SECOND * 3
        );
    }

    record StrikeInfo(List<UUID> users, int totalWeight) {
    }
    @Override
    public void httpApi(HttpServer.ApiBuilder apiBuilder) {
        apiBuilder.post("update", StrikeInfo.class, (data, request) -> {
            Main.instance().getLogger().info(String.format(
                "API Triggered Strike-Profile update for %s",
                data.users.stream().map(UUID::toString).collect(Collectors.joining(", ")))
            );
            this.processUpdate(data);
            return HttpServer.nothing;
        });

        apiBuilder.get("cords", req -> {
            Map<UUID, CordLeak.LeakInfo> raw = CordLeak.getAllLeaked();
            Map<String, LeakInfoDto> dto = new HashMap<>();
            for (Map.Entry<UUID, CordLeak.LeakInfo> entry : raw.entrySet()) {
                dto.put(entry.getKey().toString(), LeakInfoDto.from(entry.getKey(), entry.getValue()));
            }
            return dto;
        });
    }

    record LocationDto(String world, double x, double y, double z, float yaw, float pitch) {
        static LocationDto from(Location loc) {
            return new LocationDto(loc.getWorld().getName(), loc.getX(), loc.getY(), loc.getZ(), loc.getYaw(), loc.getPitch());
        }
    }

    record LeakInfoDto(String name, LocationDto location, long leakedAt) {
        static LeakInfoDto from(UUID uuid, CordLeak.LeakInfo info) {
            String name = Bukkit.getOfflinePlayer(uuid).getName();
            return new LeakInfoDto(name, LocationDto.from(info.location()), info.leakedAt().toEpochMilli());
        }
    }


    private void processUpdate(StrikeInfo data) {
        try {
            VaroTeam select = null;
            for(UUID uuid : data.users) {
                select = this.queryAppliance(Teams.class).getTeamFromPlayer(uuid);
                if(select != null) break;
            }
            Objects.requireNonNull(select);
            VaroTeam team = select;

            if(data.totalWeight < 3) {
                team.members.forEach(member -> {
                    Main.logger().info(String.format("Unbanning player %s because there are less than 3 Strikes!", member.player.getName()));
                    Bukkit.getBanList(BanListType.PROFILE).pardon(member.player.getPlayerProfile());
                });
            }

            switch(data.totalWeight) {
                case 1 -> {
                    Main.logger().info(String.format("Cord leak for Team %s", team.name));

                    ComponentBuilder<TextComponent, TextComponent.Builder> text = Component.text();
                    IteratorUtil.onlinePlayers(player -> player.sendMessage(Component.text(
                        String.format("Das Team %s hat einen Strike erhalten. Daher werden folgende Koordinaten des Teams veröffentlicht:", team.name),
                        NamedTextColor.RED
                    )));

                    data.users.forEach(uuid -> {
                        CordLeak.markAsLeaked(uuid);
                        this.appendCordLeak(text, uuid);
                    });

                    IteratorUtil.onlinePlayers(p -> p.sendMessage(text));
                }

                case 2 -> team.members.forEach(member -> {
                    Main.logger().info(String.format("Clearing Inventory of %s", member.player.getName()));
                    Optional<Player> player = Optional.ofNullable(Bukkit.getPlayer(member.player.getUniqueId()));
                    player.ifPresentOrElse(
                        p -> p.getInventory().clear(),
                        () -> InvClear.add(member.player.getUniqueId())
                    );
                });

                case 3 -> {
                    team.members.forEach(member -> {
                        Main.logger().info(String.format("Banning player %s because of third Strike!", member.player.getName()));
                        Bukkit.getBanList(BanListType.PROFILE)
                            .addBan(member.player.getPlayerProfile(), "projektausschluss", (Date) null, null);
                    });
                    Bukkit.getScheduler().runTask(Main.instance(), () -> team.getOnlinePlayers().forEach(Player::kick));
                }
            }
        } catch(Exception e) {
            Main.logger().warning("Failed to process Strikes: " + e.getMessage());
            Main.logger().throwing("Strike", "process", e);
        }
    }

    public void appendCordLeak(ComponentBuilder<TextComponent, TextComponent.Builder> text, UUID playerToLeak) {
        OfflinePlayer player = Bukkit.getOfflinePlayer(playerToLeak);
        CordLeak.LeakInfo cords = CordLeak.getLastKnownLocation(playerToLeak);
        if(cords == null) return;

        String timestamp = DateTimeFormatter.ofPattern("dd.MM HH:mm")
            .withZone(ZoneId.of("Europe/Berlin"))
            .format(cords.leakedAt());

        text.appendNewline();
        text.append(Component.text(Objects.requireNonNull(player.getName()),  NamedTextColor.DARK_AQUA));
        text.append(Component.text(": ", NamedTextColor.GRAY));
        text.append(Component.text(
            String.format(
                "(%s) %d, %d, %d - Uhrzeit: %s",
                cords.location().getWorld().getName(),
                cords.location().getBlockX(),
                cords.location().getBlockY(),
                cords.location().getBlockZ(),
                timestamp
            ),
            NamedTextColor.AQUA)
        );
    }

    public void checkJoin(@NotNull Player player) {
        if(InvClear.contains(player.getUniqueId())) {
            player.getInventory().clear();
            InvClear.remove(player.getUniqueId());
        }
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new OnStrikeActionListener());
    }
}
