package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams;

import eu.mhsl.craftattack.spawn.common.api.repositories.VaroReportRepository;
import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.api.client.ReqResp;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceListener;
import eu.mhsl.craftattack.spawn.core.util.text.DisconnectInfo;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.playTimer.PlayTimer;
import org.bukkit.event.EventHandler;
import org.bukkit.event.player.AsyncPlayerPreLoginEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;

import java.util.Objects;

class ConnectivityChangeListener extends ApplianceListener<Teams> {
    @EventHandler
    public void onLogin(AsyncPlayerPreLoginEvent event) {
        boolean result = this.getAppliance().canLogin(event.getUniqueId());
        if(!result) {
            event.kickMessage(new DisconnectInfo(
                "Kein Teilnehmer",
                "Du bist nicht als Teilnehmer registriert oder bist ausgeschieden!",
                "Sollte dies ein Fehler sein, kontaktiere bitte einen Admin.",
                event.getUniqueId()
            ).getComponent());
            event.setLoginResult(AsyncPlayerPreLoginEvent.Result.KICK_OTHER);
        }
    }

    @EventHandler
    public void onJoin(PlayerJoinEvent event) {
        this.getAppliance().enforceTeamJoin(event.getPlayer());
    }

    @EventHandler
    public void onLeave(PlayerQuitEvent event) {
        if(event.getReason().equals(PlayerQuitEvent.QuitReason.KICKED)) {
            Main.logger().info(String.format(
                "Player %s left the Server. The 'teamLeave' enforcement was skipped, since the QuitReason is 'kicked'",
                event.getPlayer().getName()
            ));
            return;
        }

        if(event.getPlayer().isOp()) return;

        VaroTeam team = Main.instance().getAppliance(Teams.class).getTeamFromPlayer(event.getPlayer().getUniqueId());
        Objects.requireNonNull(team, "Team not found for player " + event.getPlayer().getUniqueId());

        if(Main.instance().getAppliance(PlayTimer.class).isEnabled()) {
            Main.logger().info(String.format("Team %s got a Strike, because they %s left early!", team.name, event.getPlayer().getName()));

            VaroReportRepository.StrikeCreationInfo report = new VaroReportRepository.StrikeCreationInfo(
                null,
                event.getPlayer().getUniqueId(),
                "early left",
                "player left the server too early",
                null,
                null,
                1
            );
            ReqResp<Void> response = Main.instance().getRepositoryLoader().getRepository(VaroReportRepository.class).createStrike(report);
            Main.logger().info(String.format("Autostrike response for Team %s: %s", team.name, response));
        } else {
            Main.logger().info("Allow ealry leave because PlayTimer is not active!");
        }

        this.getAppliance().enforceTeamLeave(event.getPlayer());
    }
}
