package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.util.text.Countdown;
import eu.mhsl.craftattack.spawn.core.util.text.DisconnectInfo;
import eu.mhsl.craftattack.spawn.varo.appliances.internal.teamTasks.TeamTasks;
import eu.mhsl.craftattack.spawn.varo.appliances.internal.teamTasks.tasks.CountdownTeamTask;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.fightDetector.FightDetector;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.playTimer.PlayTimer;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

public class VaroTeam {
    public static class Member {
        public final OfflinePlayer player;
        public boolean isDead;

        public Member(UUID playerId, boolean isDead) {
            this.player = Bukkit.getOfflinePlayer(playerId);
            this.isDead = isDead;
        }
    }

    public String name;
    public String color;
    public List<Member> members;

    public VaroTeam(List<Member> members, String name, String color) {
        this.members = new ArrayList<>(members);
        this.name = name;
        this.color = color;
    }

    public boolean isTeamDead() {
        return this.members.stream().allMatch(member -> member.isDead);
    }

    public boolean hasMember(UUID uuid) {
        return this.members.stream()
            .anyMatch(member -> member.player.getUniqueId().equals(uuid));
    }

    public @Nullable Member getMemberById(UUID uuid) {
        return this.members.stream()
            .filter(member -> member.player.getUniqueId().equals(uuid))
            .findFirst()
            .orElse(null);
    }

    public void kickTeam() {
        this.kickTeam(new DisconnectInfo(
            "Unbekannter Fehler",
            "Verbindung wurde aufgrund eines unbekannten Grundes getrennt",
            "Falls du denkst, dass dies ein Fehler ist, melde dich bei einem Admin!",
            UUID.randomUUID()
        ));
    }

    public List<Player> getOnlinePlayers() {
        return this.members.stream()
            .map(member -> Bukkit.getPlayer(member.player.getUniqueId()))
            .filter(Objects::nonNull)
            .toList();
    }

    public void startCountDown() {
        if(!Main.instance().getAppliance(PlayTimer.class).isEnabled()) {
            Main.logger().warning(String.format("PlayTimer for %s not started, because it is deactivated!", this.name));
            return;
        }

        Main.logger().info(String.format("Starting Time countdown for Team %s with %d", this.name, PlayTimer.PLAYTIME_MINUTES));
        CountdownTeamTask countdown = new CountdownTeamTask(
            60 * PlayTimer.PLAYTIME_MINUTES,
            announcementData -> Component.text(
                String.format("Es verbleiben noch %d %s Spielzeit!", announcementData.count(), announcementData.unit()),
                NamedTextColor.RED
            ),
            component -> this.getOnlinePlayers().forEach(player -> player.sendMessage(component)),
            this::timeOverKick
        );
        countdown.setDefaultAnnouncements(count -> {
            if(count > 300) return null;
            if(count > 60 && count % 60 == 0) {
                return new Countdown.AnnouncementData(count / 60, "Minuten");
            }

            if(count <= 30 && (count <= 10 || count % 10 == 0)) {
                return new Countdown.AnnouncementData(count, "Sekunden");
            }

            return null;
        });
        countdown.start();
        Main.instance().getAppliance(TeamTasks.class).addTask(this, TeamTasks.Type.TIME_KICK, countdown);
    }

    public void kickTeam(DisconnectInfo disconnectInfo) {
        this.getOnlinePlayers()
            .forEach(player -> player.kick(disconnectInfo.getComponent()));
        Main.instance().getAppliance(TeamTasks.class).cancelTeamTasks(this);
    }

    public void timeOverKick() {
        boolean isInFight = Main.instance().getAppliance(FightDetector.class).isInFight(this);
        if(isInFight) {
            Main.logger().info(String.format("Cannot kick Team %s because it is in a fight!", this.name));
            this.getOnlinePlayers()
                .forEach(player -> player.sendMessage(
                    "Du befindest dich in einer Kampfhandlung oder in der Nähe eines gegnerischen Teams. Der Kick wird verzögert!"
                ));
            Bukkit.getScheduler().runTaskLater(
                Main.instance(),
                this::timeOverKick,
                Ticks.TICKS_PER_SECOND * 15
            );
            return;
        }

        Main.logger().info(String.format("Kicking Team %s because time is up!", this.name));

        DisconnectInfo timeOverInfo = new DisconnectInfo(
            "Die Zeit ist abgelaufen!",
            "Deine Spielzeit ist vorüber. Falls dir noch weitere Zeit zusteht kannst du jetzt eine Pause machen und anschließend erneut beitreten.",
            "Falls du Fragen hast, melde dich bitte bei einem Admin!",
            UUID.nameUUIDFromBytes("".getBytes())
        );

        this.kickTeam(timeOverInfo);
    }
}
